pro shaderSnowflake_event, event
;only timer events

widget_control, event.top, get_uvalue=object

object->updateTimer

return & end

;{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|

pro shaderSnowflake_cleanup, wBase
  
widget_control, wBase, get_uvalue=object
;destroy the object
obj_destroy,object

return & end

;{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|

pro shaderSnowflake::updateTimer

;give a random walk to the axis rotation, if you just want the snowflake to spin in 
;place then comment the next line out
self.axis = self.axis + randomn(seed,3)/10.0
self.oModel->rotate,self.axis,1 ;only one degree at a time
self.oWindow->Draw, self.oView
widget_control, self.wBase, timer=1/60.0 ;reset the timer

return & end

;{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|

function shaderSnowflake::init, _extra=extra
;initialization routine for the shaderSnowflake
compile_opt idl2

cd, sourcePath()

xdim = 500
ydim = 500

self.wBase =widget_base(title='Shader Snowflake',column=1)
;renderer has to be 0 and retain = 0 or 1, otherwise the shader is turned off           
self.wDrawId = widget_draw(self.wBase, xsize=xdim, ysize=ydim, $
                     graphics_level=2,  renderer=0, retain=0)
widget_control, self.wBase, /realize
widget_control, hourglass=1
widget_control, self.wDrawId, get_value=oWindow
self.oWindow=oWindow

;the view is set up to match the snowflake coordinates
self.oView = obj_new('IDLgrView', color=[100,100,100], zclip=[30,-30], $
                  viewplane_rect=[-20,-20,60,60], PROJECTION=2, eye=30.1)
                     
; Read the background image
read_jpeg,'starfield.jpg',snowfield, true=1
;image location and dimension match the view
oBackImage = obj_new('IDLgrImage',snowfield,dimen=[60,60],location=[-20,-20],interpolate=1)
;put this image in a separate model
oBackModel = obj_new('IDLgrModel')
oBackModel->add, oBackImage
displayText = ['Merry Christmas!','From Kling Research and Software, inc']
oFont = obj_new('IDLgrFont','Times*Bold*italic',size=20)
oText = obj_new('IDLgrText',string=displayText,render=0, $
                location=[[10,0],[10,-4]], updir=[0,1,0], $
                color=[0,255,0],font = oFont, align=0.5)
oBackModel->add,oText

self.oView->add, oBackModel

;this image gives a really nice overlay on the snowflake.
read_jpeg,'lead_tungstate_crystal_wall.jpg', EnvMap, TRUE=1

; Create environment map image
oEnvMap = OBJ_NEW('IDLgrImage', EnvMap, INTERPOLATE=1)
           
; Create the glass shader
oShader = OBJ_NEW('IDLgrShader', VERTEX_PROGRAM_FILENAME='Glass.vert', $
                    FRAGMENT_PROGRAM_FILENAME='Glass.frag')
;set the shader variables to give a glass effect
oShader->SetUniformVariable, 'eyePos', [0.0, 0.0, 0.0, 1.0]
; No displacement
oShader->SetUniformVariable, 'displace', [0.0, 0.0, 0.0, 0.0] 
oShader->SetUniformVariable, 'etaValues', [1.1, 1.08, 1.06]
oShader->SetUniformVariable, 'fresnelValues', [2.0, 2.0, 0.1]
;this one sets the image in the shader object
oShader->SetUniformVariable, 'environmentMap', oEnvMap

filename = 'snowflake.dxf'
;GET_DXF_OBJECTS is an undocumented routine in the lib/utilities directory
oDXF = GET_DXF_OBJECTS(filename, bTable, lTable, IGNOR=ignored, ERROR=error)
;get the polygon object
oObject = oDXF->get(/all)
oObject->getProperty,data=data
obj_destroy, oDXF
;GET_DXF_OBJECTS also creates two other models that we don't need but have to destroy to stop
;memory leaks
obj_destroy, lTable.model

;make the data all positive
data[0,*] = data[0,*] - min(data[0,*])
data[1,*] = data[1,*] - min(data[1,*])
data[2,*] = data[2,*] - min(data[2,*])

;setting shading=1 gives faster rendering, texture_interp does not seem to do much
oObject->setProperty,data=data, shading=1, style=2, $
                    shader=oShader, texture_interp=1

;need a model for the snowflake 
self.oModel = obj_new('IDLgrModel')  
self.oModel->Add, oObject
self.oView->add, self.oModel  
  
self.oWindow->Draw, self.oView

widget_control, self.wBase, set_uvalue=self

self.oContainer = obj_new('IDL_container')
self.oContainer->add,[self.oWindow, self.oView, self.oModel, oObject, oShader, $
                      oBackModel, oBackImage, oEnvMap]
;start the timer event
widget_control, self.wBase, timer=0.1
;initial rotation axis
self.axis = [1,1,1]

xmanager, 'shaderSnowflake', self.wBase, cleanup='shaderSnowflake_cleanup',/no_block

return,1
end

;{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|

pro shaderSnowflake::cleanup
;cleanup routine for the shaderSnowflake object

if obj_valid(self.oContainer) then obj_destroy,self.oContainer

return & end

;{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|

pro shaderSnowflake__define

;defintion routine for the shaderSnowflake object
void = {shaderSnowflake, $
        wBase : 0L, $
        wDrawId : 0L, $
        oWindow : obj_new(), $
        oView : obj_new(), $
        oModel : obj_new(), $
        oContainer : obj_new(), $
        axis : fltarr(3) }

return
end

;{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|{{:|


pro shaderSnowflake, _extra=extra
;driver routine for the shaderSnowflake visualization

oShaderSnowflake = obj_new('shaderSnowflake', _extra=extra)

return & end

